""":term:`CLI` command for importing data to coreDB."""

import ipaddress
import json
import os
from typing import TypeVar, Type, Any

import typer
import yaml
from pydantic import ValidationError

from gso.api.v1.imports import (
    SiteImportModel,
    import_site,
    RouterImportModel,
    import_router,
    IptrunkImportModel,
    import_iptrunk,
)
from gso.services.subscriptions import get_active_subscriptions_by_field_and_value

app: typer.Typer = typer.Typer()

T = TypeVar("T", SiteImportModel, RouterImportModel, IptrunkImportModel)


def read_data(filepath: str) -> dict | None:
    """Read data from a JSON or YAML file."""

    typer.echo(f"Starting import from {filepath}")
    _, file_extension = os.path.splitext(filepath)

    with open(filepath, "r") as f:
        if file_extension.lower() == ".json":
            return json.load(f)
        elif file_extension.lower() in (".yaml", ".yml"):
            return yaml.safe_load(f)
        else:
            typer.echo(f"Unsupported file format: {file_extension}")
            return


def generic_import_data(filepath: str, import_model: Type[T], import_function: callable, name_key: str):
    """Generic function to import data from a JSON or YAML file."""

    successfully_imported_data = []
    data = read_data(filepath)
    for details in data:
        details["customer"] = "GÉANT"
        typer.echo(f"Importing {name_key}: {details[name_key]}")
        try:
            initial_data = import_model(**details)
            import_function(initial_data)
            successfully_imported_data.append(getattr(initial_data, name_key))
            typer.echo(f"Successfully imported {name_key}: {getattr(initial_data, name_key)}")
        except ValidationError as e:
            typer.echo(f"Validation error: {e}")

    if successfully_imported_data:
        typer.echo(f"Successfully imported {name_key}s:")
        for item in successfully_imported_data:
            typer.echo(f"- {item}")


@app.command()
def import_sites(
    filepath: str = typer.Option(help="Path to the file containing the sites to import.", default="gso/sites.json")
):
    """
    Import sites into GSO.
    """
    # Use the import_data function to handle common import logic
    generic_import_data(filepath, SiteImportModel, import_site, "site_name")


@app.command()
def import_routers(
    filepath: str = typer.Option(
        help="Path to the file containing the routers to import.", default="gso/routers.json", show_default=True
    )
):
    """
    Import routers into GSO.
    """
    # Use the import_data function to handle common import logic
    generic_import_data(filepath, RouterImportModel, import_router, "hostname")


def get_router_subscription_id(node_name: str) -> str | None:
    """Get the subscription id for a router by its node name."""

    subscriptions = get_active_subscriptions_by_field_and_value("router_fqdn", node_name)
    if subscriptions:
        return str(subscriptions[0].subscription_id)
    else:
        return None


@app.command()
def import_iptrunks(
    filepath: str = typer.Option(
        help="Path to the file containing the routers to import.", default="gso/trunks.json", show_default=True
    )
):
    """
    Import IP trunks into GSO.
    """
    successfully_imported_data = []
    data = read_data(filepath)
    for trunk in data:
        ipv4_network_a = ipaddress.ip_network(trunk["config"]["nodeA"]["ipv4_address"], strict=False)
        ipv4_network_b = ipaddress.ip_network(trunk["config"]["nodeB"]["ipv4_address"], strict=False)
        ipv6_network_a = ipaddress.ip_network(trunk["config"]["nodeA"]["ipv6_address"], strict=False)
        ipv6_network_b = ipaddress.ip_network(trunk["config"]["nodeB"]["ipv6_address"], strict=False)
        # Check if IPv4 networks are equal
        if ipv4_network_a == ipv4_network_b:
            iptrunk_ipv4_network = ipv4_network_a
        else:
            # Handle the case where IPv4 networks are different
            typer.echo(f"Error: IPv4 networks are different for trunk {trunk['id']}.")
            continue
        # Check if IPv6 networks are the same
        if ipv6_network_a == ipv6_network_b:
            iptrunk_ipv6_network = ipv6_network_a
        else:
            # Handle the case where IPv6 networks are different
            typer.echo(f"Error: IPv6 networks are different for trunk {trunk['id']}.")
            continue

        typer.echo(
            f'Importing IP Trunk: {get_active_subscriptions_by_field_and_value("router_fqdn", trunk["config"]["nodeA"]["name"])}'
        )
        try:
            initial_data = IptrunkImportModel(
                customer="GÉANT",
                geant_s_sid=trunk["id"],
                iptrunk_type=trunk["config"]["common"]["type"],
                iptrunk_description=" ",
                iptrunk_speed=trunk["config"]["common"]["link_speed"],
                iptrunk_minimum_links=trunk["config"]["common"]["minimum_links"],
                side_a_node_id=get_router_subscription_id(trunk["config"]["nodeA"]["name"]),
                side_a_ae_iface=trunk["config"]["nodeA"]["ae_name"],
                side_a_ae_geant_a_sid=trunk["config"]["nodeA"]["port_sid"],
                side_a_ae_members=trunk["config"]["nodeA"]["members"],
                side_b_node_id=get_router_subscription_id(trunk["config"]["nodeB"]["name"]),
                side_b_ae_iface=trunk["config"]["nodeB"]["ae_name"],
                side_b_ae_geant_a_sid=trunk["config"]["nodeB"]["port_sid"],
                side_b_ae_members=trunk["config"]["nodeB"]["members"],
                iptrunk_ipv4_network=iptrunk_ipv4_network,
                iptrunk_ipv6_network=iptrunk_ipv6_network,
            )
            import_iptrunk(initial_data)
            successfully_imported_data.append(trunk["id"])
            typer.echo(f"Successfully imported IP Trunk: {trunk['id']}")
        except ValidationError as e:
            typer.echo(f"Validation error: {e}")

        if successfully_imported_data:
            typer.echo(f"Successfully imported IP Trunks:")
            for item in successfully_imported_data:
                typer.echo(f"- {item}")
